@@ -47,6 +47,7 @@ static const char * const mount_protocols[] = {
 	"cgi://",
 	">http://",
 	">https://",
+	"callback://"
 };
 
 LWS_VISIBLE void *
@@ -66,8 +67,15 @@ lws_protocol_vh_priv_zalloc(struct lws_vhost *vhost, const struct lws_protocols
 	while (n < vhost->count_protocols && &vhost->protocols[n] != prot)
 		n++;
 
-	if (n == vhost->count_protocols)
-		return NULL;
+	if (n == vhost->count_protocols) {
+		n = 0;
+		while (n < vhost->count_protocols &&
+		       strcmp(vhost->protocols[n].name, prot->name))
+			n++;
+
+		if (n == vhost->count_protocols)
+			return NULL;
+	}
 
 	vhost->protocol_vh_privs[n] = lws_zalloc(size);
 	return vhost->protocol_vh_privs[n];
@@ -85,8 +93,15 @@ lws_protocol_vh_priv_get(struct lws_vhost *vhost, const struct lws_protocols *pr
 		n++;
 
 	if (n == vhost->count_protocols) {
-		lwsl_err("%s: unknown protocol %p\n", __func__, prot);
-		return NULL;
+		n = 0;
+		while (n < vhost->count_protocols &&
+		       strcmp(vhost->protocols[n].name, prot->name))
+			n++;
+
+		if (n == vhost->count_protocols) {
+			lwsl_err("%s: unknown protocol %p\n", __func__, prot);
+			return NULL;
+		}
 	}
 
 	return vhost->protocol_vh_privs[n];
@@ -107,7 +122,11 @@ lws_vhost_protocol_options(struct lws_vhost *vh, const char *name)
 	return NULL;
 }
 
-int
+/*
+ * inform every vhost that hasn't already done it, that
+ * his protocols are initializing
+ */
+LWS_VISIBLE int
 lws_protocol_init(struct lws_context *context)
 {
 	struct lws_vhost *vh = context->vhost_list;
@@ -118,16 +137,21 @@ lws_protocol_init(struct lws_context *context)
 	memset(&wsi, 0, sizeof(wsi));
 	wsi.context = context;
 
-	lwsl_notice("%s\n", __func__);
+	lwsl_info("%s\n", __func__);
 
 	while (vh) {
 		wsi.vhost = vh;
 
+		/* only do the protocol init once for a given vhost */
+		if (vh->created_vhost_protocols)
+			goto next;
+
 		/* initialize supported protocols on this vhost */
 
 		for (n = 0; n < vh->count_protocols; n++) {
 			wsi.protocol = &vh->protocols[n];
-
+			if (!vh->protocols[n].name)
+				continue;
 			pvo = lws_vhost_protocol_options(vh,
 							 vh->protocols[n].name);
 			if (pvo) {
@@ -151,6 +175,13 @@ lws_protocol_init(struct lws_context *context)
 						   vh->protocols[n].name);
 						vh->default_protocol_index = n;
 					}
+					if (!strcmp(pvo->name, "raw")) {
+						lwsl_notice("Setting raw "
+						   "protocol for vh %s to %s\n",
+						   vh->name,
+						   vh->protocols[n].name);
+						vh->raw_protocol_index = n;
+					}
 					pvo = pvo->next;
 				}
 
@@ -164,23 +195,107 @@ lws_protocol_init(struct lws_context *context)
 			 * NOTE the wsi is all zeros except for the context, vh and
 			 * protocol ptrs so lws_get_context(wsi) etc can work
 			 */
-			vh->protocols[n].callback(&wsi,
+			if (vh->protocols[n].callback(&wsi,
 				LWS_CALLBACK_PROTOCOL_INIT, NULL,
-				(void *)pvo, 0);
+				(void *)pvo, 0))
+				return 1;
 		}
 
+		vh->created_vhost_protocols = 1;
+next:
 		vh = vh->vhost_next;
 	}
 
+	if (!context->protocol_init_done)
+		lws_finalize_startup(context);
+
 	context->protocol_init_done = 1;
 
 	return 0;
 }
 
-static int callback_http_dummy(
-		struct lws *wsi, enum lws_callback_reasons reason, void *user,
-		  void *in, size_t len)
+LWS_VISIBLE int
+lws_callback_http_dummy(struct lws *wsi, enum lws_callback_reasons reason,
+		    void *user, void *in, size_t len)
 {
+#ifdef LWS_WITH_CGI
+	struct lws_cgi_args *args;
+	char buf[128];
+	int n;
+#endif
+
+	switch (reason) {
+	case LWS_CALLBACK_HTTP:
+#ifndef LWS_NO_SERVER
+		if (lws_return_http_status(wsi, HTTP_STATUS_NOT_FOUND, NULL))
+			return -1;
+
+		if (lws_http_transaction_completed(wsi))
+#endif
+			return -1;
+		break;
+
+	case LWS_CALLBACK_HTTP_WRITEABLE:
+#ifdef LWS_WITH_CGI
+		if (wsi->reason_bf & 1) {
+			if (lws_cgi_write_split_stdout_headers(wsi) < 0)
+				return -1;
+
+			wsi->reason_bf &= ~1;
+			break;
+		}
+#endif
+
+		break;
+
+#ifdef LWS_WITH_CGI
+	/* CGI IO events (POLLIN/OUT) appear here, our default policy is:
+	 *
+	 *  - POST data goes on subprocess stdin
+	 *  - subprocess stdout goes on http via writeable callback
+	 *  - subprocess stderr goes to the logs
+	 */
+	case LWS_CALLBACK_CGI:
+		args = (struct lws_cgi_args *)in;
+		switch (args->ch) { /* which of stdin/out/err ? */
+		case LWS_STDIN:
+			/* TBD stdin rx flow control */
+			break;
+		case LWS_STDOUT:
+			wsi->reason_bf |= 1;
+			/* when writing to MASTER would not block */
+			lws_callback_on_writable(wsi);
+			break;
+		case LWS_STDERR:
+			n = read(lws_get_socket_fd(args->stdwsi[LWS_STDERR]),
+						   buf, sizeof(buf) - 2);
+			if (n > 0) {
+				if (buf[n - 1] != '\n')
+					buf[n++] = '\n';
+				buf[n] = '\0';
+				lwsl_notice("CGI-stderr: %s\n", buf);
+			}
+			break;
+		}
+		break;
+
+	case LWS_CALLBACK_CGI_TERMINATED:
+		return -1;
+
+	case LWS_CALLBACK_CGI_STDIN_DATA:  /* POST body for stdin */
+		args = (struct lws_cgi_args *)in;
+		args->data[args->len] = '\0';
+		n = write(lws_get_socket_fd(args->stdwsi[LWS_STDIN]),
+			  args->data, args->len);
+		if (n < args->len)
+			lwsl_notice("LWS_CALLBACK_CGI_STDIN_DATA: "
+				    "sent %d only %d went", n, args->len);
+		return n;
+#endif
+	default:
+		break;
+	}
+
 	return 0;
 }
 
@@ -191,16 +306,21 @@ static const struct lws_protocols protocols_dummy[] = {
 
 	{
 		"http-only",		/* name */
-		callback_http_dummy,		/* callback */
+		lws_callback_http_dummy,		/* callback */
 		0,	/* per_session_data_size */
 		0,			/* max frame size / rx buffer */
+		0, NULL
 	},
 	/*
 	 * the other protocols are provided by lws plugins
 	 */
-	{ NULL, NULL, 0, 0 } /* terminator */
+	{ NULL, NULL, 0, 0, 0, NULL} /* terminator */
 };
 
+#ifdef LWS_PLAT_OPTEE
+#undef LWS_HAVE_GETENV
+#endif
+
 LWS_VISIBLE struct lws_vhost *
 lws_create_vhost(struct lws_context *context,
 		 struct lws_context_creation_info *info)
@@ -208,12 +328,16 @@ lws_create_vhost(struct lws_context *context,
 	struct lws_vhost *vh = lws_zalloc(sizeof(*vh)),
 			 **vh1 = &context->vhost_list;
 	const struct lws_http_mount *mounts;
+	const struct lws_protocol_vhost_options *pvo;
 #ifdef LWS_WITH_PLUGINS
 	struct lws_plugin *plugin = context->plugin_list;
-	struct lws_protocols *lwsp;
-	int m, n, f = !info->pvo;
 #endif
+	struct lws_protocols *lwsp;
+	int m, f = !info->pvo;
+#ifdef LWS_HAVE_GETENV
 	char *p;
+#endif
+	int n;
 
 	if (!vh)
 		return NULL;
@@ -235,31 +359,37 @@ lws_create_vhost(struct lws_context *context,
 
 	vh->options = info->options;
 	vh->pvo = info->pvo;
-	vh->keepalive_timeout = info->keepalive_timeout;
+	vh->headers = info->headers;
+	if (info->keepalive_timeout)
+		vh->keepalive_timeout = info->keepalive_timeout;
+	else
+		vh->keepalive_timeout = 5;
 
+	/*
+	 * give the vhost a unified list of protocols including the
+	 * ones that came from plugins
+	 */
+	lwsp = lws_zalloc(sizeof(struct lws_protocols) *
+				   (vh->count_protocols +
+				   context->plugin_protocol_count + 1));
+	if (!lwsp) {
+		lwsl_err("OOM\n");
+		return NULL;
+	}
+
+	m = vh->count_protocols;
+	memcpy(lwsp, info->protocols, sizeof(struct lws_protocols) * m);
+
+	/* for compatibility, all protocols enabled on vhost if only
+	 * the default vhost exists.  Otherwise only vhosts who ask
+	 * for a protocol get it enabled.
+	 */
+
+	if (info->options & LWS_SERVER_OPTION_EXPLICIT_VHOSTS)
+		f = 0;
+	(void)f;
 #ifdef LWS_WITH_PLUGINS
 	if (plugin) {
-		/*
-		 * give the vhost a unified list of protocols including the
-		 * ones that came from plugins
-		 */
-		lwsp = lws_zalloc(sizeof(struct lws_protocols) *
-					   (vh->count_protocols +
-					   context->plugin_protocol_count + 1));
-		if (!lwsp)
-			return NULL;
-
-		m = vh->count_protocols;
-		memcpy(lwsp, info->protocols,
-		       sizeof(struct lws_protocols) * m);
-
-		/* for compatibility, all protocols enabled on vhost if only
-		 * the default vhost exists.  Otherwise only vhosts who ask
-		 * for a protocol get it enabled.
-		 */
-
-		if (info->options & LWS_SERVER_OPTION_EXPLICIT_VHOSTS)
-			f = 0;
 
 		while (plugin) {
 			for (n = 0; n < plugin->caps.count_protocols; n++) {
@@ -278,10 +408,19 @@ lws_create_vhost(struct lws_context *context,
 			}
 			plugin = plugin->list;
 		}
-		vh->protocols = lwsp;
-	} else
+	}
 #endif
+
+	if (
+#ifdef LWS_WITH_PLUGINS
+	    (context->plugin_list) ||
+#endif
+	    info->options & LWS_SERVER_OPTION_EXPLICIT_VHOSTS)
+		vh->protocols = lwsp;
+	else {
 		vh->protocols = info->protocols;
+		free(lwsp);
+	}
 
 	vh->same_vh_protocol_list = (struct lws **)
 			lws_zalloc(sizeof(struct lws *) * vh->count_protocols);
@@ -294,14 +433,29 @@ lws_create_vhost(struct lws_context *context,
 				vh->name, info->iface, vh->count_protocols);
 	} else
 #endif
-	lwsl_notice("Creating Vhost '%s' port %d, %d protocols\n",
-			vh->name, info->port, vh->count_protocols);
+	lwsl_notice("Creating Vhost '%s' port %d, %d protocols, IPv6 %s\n",
+			vh->name, info->port, vh->count_protocols, LWS_IPV6_ENABLED(vh) ? "on" : "off");
 
 	mounts = info->mounts;
 	while (mounts) {
 		lwsl_notice("   mounting %s%s to %s\n",
 				mount_protocols[mounts->origin_protocol],
 				mounts->origin, mounts->mountpoint);
+
+		/* convert interpreter protocol names to pointers */
+		pvo = mounts->interpret;
+		while (pvo) {
+			for (n = 0; n < vh->count_protocols; n++)
+				if (!strcmp(pvo->value, vh->protocols[n].name)) {
+					((struct lws_protocol_vhost_options *)pvo)->value =
+							(const char *)(long)n;
+					break;
+				}
+			if (n == vh->count_protocols)
+				lwsl_err("ignoring unknown interpret protocol %s\n", pvo->value);
+			pvo = pvo->next;
+		}
+
 		mounts = mounts->mount_next;
 	}
 
@@ -340,6 +494,7 @@ lws_create_vhost(struct lws_context *context,
 #endif
 
 	vh->listen_port = info->port;
+#if !defined(LWS_WITH_ESP8266)
 	vh->http_proxy_port = 0;
 	vh->http_proxy_address[0] = '\0';
 
@@ -357,7 +512,7 @@ lws_create_vhost(struct lws_context *context,
 			lws_set_proxy(vh, p);
 #endif
 	}
-
+#endif
 	vh->ka_time = info->ka_time;
 	vh->ka_interval = info->ka_interval;
 	vh->ka_probes = info->ka_probes;
@@ -368,7 +523,7 @@ lws_create_vhost(struct lws_context *context,
 #ifdef LWS_WITH_ACCESS_LOG
 	if (info->log_filepath) {
 		vh->log_fd = open(info->log_filepath, O_CREAT | O_APPEND | O_RDWR, 0600);
-		if (vh->log_fd == LWS_INVALID_FILE) {
+		if (vh->log_fd == (int)LWS_INVALID_FILE) {
 			lwsl_err("unable to open log filepath %s\n",
 				 info->log_filepath);
 			goto bail;
@@ -381,17 +536,16 @@ lws_create_vhost(struct lws_context *context,
 						info->log_filepath);
 #endif
 	} else
-		vh->log_fd = LWS_INVALID_FILE;
+		vh->log_fd = (int)LWS_INVALID_FILE;
 #endif
-
 	if (lws_context_init_server_ssl(info, vh))
 		goto bail;
-
 	if (lws_context_init_client_ssl(info, vh))
 		goto bail;
-
-	if (lws_context_init_server(info, vh))
+	if (lws_context_init_server(info, vh)) {
+		lwsl_err("init server failed\n");
 		goto bail;
+	}
 
 	while (1) {
 		if (!(*vh1)) {
@@ -400,6 +554,10 @@ lws_create_vhost(struct lws_context *context,
 		}
 		vh1 = &(*vh1)->vhost_next;
 	};
+	/* for the case we are adding a vhost much later, after server init */
+
+	if (context->protocol_init_done)
+		lws_protocol_init(context);
 
 	return vh;
 
@@ -409,40 +567,23 @@ bail:
 	return NULL;
 }
 
-/**
- * lws_create_context() - Create the websocket handler
- * @info:	pointer to struct with parameters
- *
- *	This function creates the listening socket (if serving) and takes care
- *	of all initialization in one step.
- *
- *	After initialization, it returns a struct lws_context * that
- *	represents this server.  After calling, user code needs to take care
- *	of calling lws_service() with the context pointer to get the
- *	server's sockets serviced.  This must be done in the same process
- *	context as the initialization call.
- *
- *	The protocol callback functions are called for a handful of events
- *	including http requests coming in, websocket connections becoming
- *	established, and data arriving; it's also called periodically to allow
- *	async transmission.
- *
- *	HTTP requests are sent always to the FIRST protocol in @protocol, since
- *	at that time websocket protocol has not been negotiated.  Other
- *	protocols after the first one never see any HTTP callack activity.
- *
- *	The server created is a simple http server by default; part of the
- *	websocket standard is upgrading this http connection to a websocket one.
- *
- *	This allows the same server to provide files like scripts and favicon /
- *	images or whatever over http and dynamic data over websockets all in
- *	one place; they're all handled in the user callback.
- */
+LWS_VISIBLE int
+lws_init_vhost_client_ssl(const struct lws_context_creation_info *info,
+			  struct lws_vhost *vhost)
+{
+	struct lws_context_creation_info i;
+
+	memcpy(&i, info, sizeof(i));
+	i.port = CONTEXT_PORT_NO_LISTEN;
+
+	return lws_context_init_client_ssl(&i, vhost);
+}
+
 LWS_VISIBLE struct lws_context *
 lws_create_context(struct lws_context_creation_info *info)
 {
 	struct lws_context *context = NULL;
-	struct lws wsi;
+	struct lws_plat_file_ops *prev;
 #ifndef LWS_NO_DAEMONIZE
 	int pid_daemon = get_daemonize_pid();
 #endif
@@ -453,6 +594,9 @@ lws_create_context(struct lws_context_creation_info *info)
 
 	lwsl_notice("Initial logging level %d\n", log_level);
 	lwsl_notice("Libwebsockets version: %s\n", library_version);
+#if defined(GCC_VER)
+	lwsl_notice("Compiled with  %s\n", GCC_VER);
+#endif
 #if LWS_POSIX
 #ifdef LWS_USE_IPV6
 	if (!lws_check_opt(info->options, LWS_SERVER_OPTION_DISABLE_IPV6))
@@ -462,14 +606,16 @@ lws_create_context(struct lws_context_creation_info *info)
 #else
 	lwsl_notice("IPV6 not compiled in\n");
 #endif
+#if !defined(LWS_PLAT_OPTEE) && !defined(LWS_PLAT_ESP32)
 	lws_feature_status_libev(info);
 	lws_feature_status_libuv(info);
+#endif
 #endif
 	lwsl_info(" LWS_DEF_HEADER_LEN    : %u\n", LWS_DEF_HEADER_LEN);
 	lwsl_info(" LWS_MAX_PROTOCOLS     : %u\n", LWS_MAX_PROTOCOLS);
 	lwsl_info(" LWS_MAX_SMP           : %u\n", LWS_MAX_SMP);
 	lwsl_info(" SPEC_LATEST_SUPPORTED : %u\n", SPEC_LATEST_SUPPORTED);
-	lwsl_info(" sizeof (*info)        : %u\n", sizeof(*info));
+	lwsl_info(" sizeof (*info)        : %ld\n", (long)sizeof(*info));
 #if LWS_POSIX
 	lwsl_info(" SYSTEM_RANDOM_FILEPATH: '%s'\n", SYSTEM_RANDOM_FILEPATH);
 #endif
@@ -481,8 +627,49 @@ lws_create_context(struct lws_context_creation_info *info)
 		lwsl_err("No memory for websocket context\n");
 		return NULL;
 	}
+	if (info->pt_serv_buf_size)
+		context->pt_serv_buf_size = info->pt_serv_buf_size;
+	else
+		context->pt_serv_buf_size = 4096;
+
+	/* default to just the platform fops implementation */
+
+	context->fops_platform.LWS_FOP_OPEN	= _lws_plat_file_open;
+	context->fops_platform.LWS_FOP_CLOSE	= _lws_plat_file_close;
+	context->fops_platform.LWS_FOP_SEEK_CUR	= _lws_plat_file_seek_cur;
+	context->fops_platform.LWS_FOP_READ	= _lws_plat_file_read;
+	context->fops_platform.LWS_FOP_WRITE	= _lws_plat_file_write;
+	context->fops_platform.fi[0].sig	= NULL;
+
+	/*
+	 *  arrange a linear linked-list of fops starting from context->fops
+	 *
+	 * platform fops
+	 * [ -> fops_zip (copied into context so .next settable) ]
+	 * [ -> info->fops ]
+	 */
+
+	context->fops = &context->fops_platform;
+	prev = (struct lws_plat_file_ops *)context->fops;
+
+#if defined(LWS_WITH_ZIP_FOPS)
+	/* make a soft copy so we can set .next */
+	context->fops_zip = fops_zip;
+	prev->next = &context->fops_zip;
+	prev = (struct lws_plat_file_ops *)prev->next;
+#endif
+
+	/* if user provided fops, tack them on the end of the list */
+	if (info->fops)
+		prev->next = info->fops;
+
+	context->reject_service_keywords = info->reject_service_keywords;
+	if (info->external_baggage_free_on_destroy)
+		context->external_baggage_free_on_destroy =
+			info->external_baggage_free_on_destroy;
 
 	context->time_up = time(NULL);
+
 #ifndef LWS_NO_DAEMONIZE
 	if (pid_daemon) {
 		context->started_with_parent = pid_daemon;
@@ -517,12 +704,18 @@ lws_create_context(struct lws_context_creation_info *info)
 	else
 		context->timeout_secs = AWAITING_TIMEOUT;
 
+	context->ws_ping_pong_interval = info->ws_ping_pong_interval;
+
 	lwsl_info(" default timeout (secs): %u\n", context->timeout_secs);
 
 	if (info->max_http_header_data)
 		context->max_http_header_data = info->max_http_header_data;
 	else
-		context->max_http_header_data = LWS_DEF_HEADER_LEN;
+		if (info->max_http_header_data2)
+			context->max_http_header_data =
+					info->max_http_header_data2;
+		else
+			context->max_http_header_data = LWS_DEF_HEADER_LEN;
 	if (info->max_http_header_pool)
 		context->max_http_header_pool = info->max_http_header_pool;
 	else
@@ -533,13 +726,15 @@ lws_create_context(struct lws_context_creation_info *info)
 	 * and header data pool
 	 */
 	for (n = 0; n < context->count_threads; n++) {
-		context->pt[n].serv_buf = lws_zalloc(LWS_MAX_SOCKET_IO_BUF);
+		context->pt[n].serv_buf = lws_zalloc(context->pt_serv_buf_size);
 		if (!context->pt[n].serv_buf) {
 			lwsl_err("OOM\n");
 			return NULL;
 		}
 
+#ifdef LWS_USE_LIBUV
 		context->pt[n].context = context;
+#endif
 		context->pt[n].tid = n;
 		context->pt[n].http_header_data = lws_malloc(context->max_http_header_data *
 						       context->max_http_header_pool);
@@ -567,9 +762,6 @@ lws_create_context(struct lws_context_creation_info *info)
 	lwsl_notice(" Threads: %d each %d fds\n", context->count_threads,
 		    context->fd_limit_per_thread);
 
-	memset(&wsi, 0, sizeof(wsi));
-	wsi.context = context;
-
 	if (!info->ka_interval && info->ka_time > 0) {
 		lwsl_err("info->ka_interval can't be 0 if ka_time used\n");
 		return NULL;
@@ -596,20 +788,20 @@ lws_create_context(struct lws_context_creation_info *info)
 	context->lws_uv_sigint_cb = &lws_uv_sigint_cb;
 #endif
 
-	lwsl_info(" mem: context:         %5u bytes (%d ctx + (%d thr x %d))\n",
-		  sizeof(struct lws_context) +
-		  (context->count_threads * LWS_MAX_SOCKET_IO_BUF),
-		  sizeof(struct lws_context),
-		  context->count_threads,
-		  LWS_MAX_SOCKET_IO_BUF);
+	lwsl_info(" mem: context:         %5lu bytes (%ld ctx + (%ld thr x %d))\n",
+		  (long)sizeof(struct lws_context) +
+		  (context->count_threads * context->pt_serv_buf_size),
+		  (long)sizeof(struct lws_context),
+		  (long)context->count_threads,
+		  context->pt_serv_buf_size);
 
-	lwsl_info(" mem: http hdr rsvd:   %5u bytes (%u thr x (%u + %u) x %u))\n",
-		    (context->max_http_header_data +
+	lwsl_info(" mem: http hdr rsvd:   %5lu bytes (%u thr x (%u + %lu) x %u))\n",
+		    (long)(context->max_http_header_data +
 		     sizeof(struct allocated_headers)) *
 		    context->max_http_header_pool * context->count_threads,
 		    context->count_threads,
 		    context->max_http_header_data,
-		    sizeof(struct allocated_headers),
+		    (long)sizeof(struct allocated_headers),
 		    context->max_http_header_pool);
 	n = sizeof(struct lws_pollfd) * context->count_threads *
 	    context->fd_limit_per_thread;
@@ -624,9 +816,6 @@ lws_create_context(struct lws_context_creation_info *info)
 		context->server_string = info->server_string;
 		context->server_string_len = (short)
 				strlen(context->server_string);
-	} else {
-		context->server_string = "libwebsockets";
-		context->server_string_len = 13;
 	}
 
 #if LWS_MAX_SMP > 1
@@ -655,8 +844,8 @@ lws_create_context(struct lws_context_creation_info *info)
 
 	lws_context_init_extensions(info, context);
 
-	lwsl_notice(" mem: per-conn:        %5u bytes + protocol rx buf\n",
-		    sizeof(struct lws));
+	lwsl_notice(" mem: per-conn:        %5lu bytes + protocol rx buf\n",
+		    (unsigned long)sizeof(struct lws));
 
 	strcpy(context->canonical_hostname, "unknown");
 	lws_server_get_canonical_hostname(context, info);
@@ -692,30 +881,79 @@ bail:
 	return NULL;
 }
 
-/**
- * lws_context_destroy() - Destroy the websocket context
- * @context:	Websocket context
- *
- *	This function closes any active connections and then frees the
- *	context.  After calling this, any further use of the context is
- *	undefined.
- */
+LWS_VISIBLE LWS_EXTERN void
+lws_context_deprecate(struct lws_context *context, lws_reload_func cb)
+{
+	struct lws_vhost *vh = context->vhost_list, *vh1;
+	struct lws *wsi;
+
+	/*
+	 * "deprecation" means disable the context from accepting any new
+	 * connections and free up listen sockets to be used by a replacement
+	 * context.
+	 *
+	 * Otherwise the deprecated context remains operational, until its
+	 * number of connected sockets falls to zero, when it is deleted.
+	 */
+
+	/* for each vhost, close his listen socket */
+
+	while (vh) {
+		wsi = vh->lserv_wsi;
+		if (wsi) {
+			wsi->socket_is_permanently_unusable = 1;
+			lws_close_free_wsi(wsi, LWS_CLOSE_STATUS_NOSTATUS);
+			wsi->context->deprecation_pending_listen_close_count++;
+			/*
+			 * other vhosts can share the listen port, they
+			 * point to the same wsi.  So zap those too.
+			 */
+			vh1 = context->vhost_list;
+			while (vh1) {
+				if (vh1->lserv_wsi == wsi)
+					vh1->lserv_wsi = NULL;
+				vh1 = vh1->vhost_next;
+			}
+		}
+		vh = vh->vhost_next;
+	}
+
+	context->deprecated = 1;
+	context->deprecation_cb = cb;
+}
+
+LWS_VISIBLE LWS_EXTERN int
+lws_context_is_deprecated(struct lws_context *context)
+{
+	return context->deprecated;
+}
+
+LWS_VISIBLE void
+lws_context_destroy2(struct lws_context *context);
+
 LWS_VISIBLE void
 lws_context_destroy(struct lws_context *context)
 {
 	const struct lws_protocols *protocol = NULL;
 	struct lws_context_per_thread *pt;
-	struct lws_vhost *vh = NULL, *vh1;
+	struct lws_vhost *vh = NULL;
 	struct lws wsi;
 	int n, m;
 
-	lwsl_notice("%s\n", __func__);
-
-	if (!context)
+	if (!context) {
+		lwsl_notice("%s: ctx %p\n", __func__, context);
 		return;
+	}
+	if (context->being_destroyed1) {
+		lwsl_notice("%s: ctx %p: already being destroyed\n", __func__, context);
+		return;
+	}
+
+	lwsl_notice("%s: ctx %p\n", __func__, context);
 
 	m = context->count_threads;
 	context->being_destroyed = 1;
+	context->being_destroyed1 = 1;
 
 	memset(&wsi, 0, sizeof(wsi));
 	wsi.context = context;
@@ -740,6 +978,7 @@ lws_context_destroy(struct lws_context *context)
 		}
 		lws_pt_mutex_destroy(pt);
 	}
+
 	/*
 	 * give all extensions a chance to clean up any per-context
 	 * allocations they might have made
@@ -789,12 +1028,30 @@ lws_context_destroy(struct lws_context *context)
 			lws_free(pt->http_header_data);
 	}
 	lws_plat_context_early_destroy(context);
-	lws_ssl_context_destroy(context);
 
 	if (context->pt[0].fds)
 		lws_free_set_NULL(context->pt[0].fds);
 
-	/* free all the vhost allocations */
+	if (!LWS_LIBUV_ENABLED(context))
+		lws_context_destroy2(context);
+}
+
+/*
+ * call the second one after the event loop has been shut down cleanly
+ */
+
+LWS_VISIBLE void
+lws_context_destroy2(struct lws_context *context)
+{
+	const struct lws_protocols *protocol = NULL;
+	struct lws_vhost *vh = NULL, *vh1;
+	int n;
+
+	lwsl_notice("%s: ctx %p\n", __func__, context);
+
+	/*
+	 * free all the per-vhost allocations
+	 */
 
 	vh = context->vhost_list;
 	while (vh) {
@@ -804,6 +1061,7 @@ lws_context_destroy(struct lws_context *context)
 			while (n < vh->count_protocols) {
 				if (vh->protocol_vh_privs &&
 				    vh->protocol_vh_privs[n]) {
+					// lwsl_notice("   %s: freeing per-vhost protocol data %p\n", __func__, vh->protocol_vh_privs[n]);
 					lws_free(vh->protocol_vh_privs[n]);
 					vh->protocol_vh_privs[n] = NULL;
 				}
@@ -818,13 +1076,18 @@ lws_context_destroy(struct lws_context *context)
 #ifdef LWS_WITH_PLUGINS
 		if (context->plugin_list)
 			lws_free((void *)vh->protocols);
+#else
+		if (vh->options & LWS_SERVER_OPTION_EXPLICIT_VHOSTS)
+			lws_free((void *)vh->protocols);
+#endif
+#ifdef LWS_WITH_PLUGINS
 #ifndef LWS_NO_EXTENSIONS
 		if (context->plugin_extension_count)
 			lws_free((void *)vh->extensions);
 #endif
 #endif
 #ifdef LWS_WITH_ACCESS_LOG
-		if (vh->log_fd != LWS_INVALID_FILE)
+		if (vh->log_fd != (int)LWS_INVALID_FILE)
 			close(vh->log_fd);
 #endif
 
@@ -833,7 +1096,12 @@ lws_context_destroy(struct lws_context *context)
 		vh = vh1;
 	}
 
+	lws_ssl_context_destroy(context);
 	lws_plat_context_late_destroy(context);
 
+	if (context->external_baggage_free_on_destroy)
+		free(context->external_baggage_free_on_destroy);
+
+
 	lws_free(context);
 }
