@@ -1,6 +1,10 @@
 /* set of parsable strings -- ALL LOWER CASE */
 
-static const char *set[] = {
+#if !defined(STORE_IN_ROM)
+#define STORE_IN_ROM
+#endif
+
+STORE_IN_ROM static const char * const set[] = {
 	"get ",
 	"post ",
 	"options ",
@@ -90,6 +94,9 @@ static const char *set[] = {
 	"x-real-ip:",
 	"http/1.0 ",
 
+	"x-forwarded-for",
+	"connect ",
+
 	"", /* not matchable */
 
 };
