@@ -27,9 +27,18 @@ lws_calllback_as_writeable(struct lws *wsi)
 	int n;
 
 	switch (wsi->mode) {
+	case LWSCM_RAW:
+		n = LWS_CALLBACK_RAW_WRITEABLE;
+		break;
+	case LWSCM_RAW_FILEDESC:
+		n = LWS_CALLBACK_RAW_WRITEABLE_FILE;
+		break;
 	case LWSCM_WS_CLIENT:
 		n = LWS_CALLBACK_CLIENT_WRITEABLE;
 		break;
+	case LWSCM_WSCL_ISSUE_HTTP_BODY:
+		n = LWS_CALLBACK_CLIENT_HTTP_WRITEABLE;
+		break;
 	case LWSCM_WS_SERVING:
 		n = LWS_CALLBACK_SERVER_WRITEABLE;
 		break;
@@ -76,6 +85,9 @@ lws_handle_POLLOUT_event(struct lws *wsi, struct lws_pollfd *pollfd)
 		if (wsi->state == LWSS_FLUSHING_STORED_SEND_BEFORE_CLOSE)
 			return -1; /* retry closing now */
 
+	if (wsi->mode == LWSCM_WSCL_ISSUE_HTTP_BODY)
+		goto user_service;
+
 
 #ifdef LWS_USE_HTTP2
 	/* Priority 2: protocol packets
@@ -127,6 +139,29 @@ lws_handle_POLLOUT_event(struct lws *wsi, struct lws_pollfd *pollfd)
 		return 0;
 	}
 
+	if (wsi->state == LWSS_ESTABLISHED &&
+	    !wsi->socket_is_permanently_unusable &&
+	    wsi->u.ws.send_check_ping) {
+
+		lwsl_info("issuing ping on wsi %p\n", wsi);
+		wsi->u.ws.send_check_ping = 0;
+		n = lws_write(wsi, &wsi->u.ws.ping_payload_buf[LWS_PRE],
+			      0, LWS_WRITE_PING);
+		if (n < 0)
+			return -1;
+
+		/*
+		 * we apparently were able to send the PING in a reasonable time
+		 * now reset the clock on our peer to be able to send the
+		 * PONG in a reasonable time.
+		 */
+
+		lws_set_timeout(wsi, PENDING_TIMEOUT_WS_PONG_CHECK_GET_PONG,
+				wsi->context->timeout_secs);
+
+		return 0;
+	}
+
 	/* Priority 4: if we are closing, not allowed to send more data frags
 	 *	       which means user callback or tx ext flush banned now
 	 */
@@ -166,6 +201,8 @@ lws_handle_POLLOUT_event(struct lws *wsi, struct lws_pollfd *pollfd)
 	 */
 
 	ret = 1;
+	if (wsi->mode == LWSCM_RAW || wsi->mode == LWSCM_RAW_FILEDESC)
+		ret = 0;
 	while (ret == 1) {
 
 		/* default to nobody has more to spill */
@@ -203,7 +240,7 @@ lws_handle_POLLOUT_event(struct lws *wsi, struct lws_pollfd *pollfd)
 			 * Keep amount spilled small to minimize chance of this
 			 */
 			if (n != eff_buf.token_len) {
-				lwsl_err("Unable to spill ext %d vs %s\n",
+				lwsl_err("Unable to spill ext %d vs %d\n",
 							  eff_buf.token_len, n);
 				return -1;
 			}
@@ -247,8 +284,8 @@ user_service:
 			return 1;
 		}
 
-
-	if (!wsi->hdr_parsing_completed)
+	if (wsi->mode != LWSCM_WSCL_ISSUE_HTTP_BODY &&
+	    !wsi->hdr_parsing_completed)
 		return 0;
 
 #ifdef LWS_WITH_CGI
@@ -309,7 +346,12 @@ notify:
 int
 lws_service_timeout_check(struct lws *wsi, unsigned int sec)
 {
+//#if LWS_POSIX
 	struct lws_context_per_thread *pt = &wsi->context->pt[(int)wsi->tsi];
+	int n = 0;
+//#endif
+
+	(void)n;
 
 	/*
 	 * if extensions want in on it (eg, we are a mux parent)
@@ -326,15 +368,17 @@ lws_service_timeout_check(struct lws *wsi, unsigned int sec)
 	 * connection
 	 */
 	if ((time_t)sec > wsi->pending_timeout_limit) {
-#if LWS_POSIX
+//#if LWS_POSIX
+		if (wsi->desc.sockfd != LWS_SOCK_INVALID && wsi->position_in_fds_table >= 0)
+			n = pt->fds[wsi->position_in_fds_table].events;
+
 		/* no need to log normal idle keepalive timeout */
 		if (wsi->pending_timeout != PENDING_TIMEOUT_HTTP_KEEPALIVE_IDLE)
-			lwsl_notice("wsi %p: TIMEDOUT WAITING on %d (did hdr %d, ah %p, wl %d, pfd events %d)\n",
+			lwsl_notice("wsi %p: TIMEDOUT WAITING on %d (did hdr %d, ah %p, wl %d, pfd events %d) %llu vs %llu\n",
 			    (void *)wsi, wsi->pending_timeout,
 			    wsi->hdr_parsing_completed, wsi->u.hdr.ah,
-			    pt->ah_wait_list_length,
-			    pt->fds[wsi->sock].events);
-#endif
+			    pt->ah_wait_list_length, n, (unsigned long long)sec, (unsigned long long)wsi->pending_timeout_limit);
+//#endif
 		/*
 		 * Since he failed a timeout, he already had a chance to do
 		 * something and was unable to... that includes situations like
@@ -346,7 +390,7 @@ lws_service_timeout_check(struct lws *wsi, unsigned int sec)
 		if (wsi->mode == LWSCM_WSCL_WAITING_SSL)
 			wsi->vhost->protocols[0].callback(wsi,
 				LWS_CALLBACK_CLIENT_CONNECTION_ERROR,
-				wsi->user_space, NULL, 0);
+				wsi->user_space, (void *)"Timed out waiting SSL", 21);
 
 		lws_close_free_wsi(wsi, LWS_CLOSE_STATUS_NOSTATUS);
 
@@ -381,7 +425,7 @@ int lws_rxflow_cache(struct lws *wsi, unsigned char *buf, int n, int len)
  * activity in poll() when we have something that already needs service
  */
 
-int
+LWS_VISIBLE LWS_EXTERN int
 lws_service_adjust_timeout(struct lws_context *context, int timeout_ms, int tsi)
 {
 	struct lws_context_per_thread *pt = &context->pt[tsi];
@@ -445,9 +489,10 @@ lws_service_flag_pending(struct lws_context *context, int tsi)
 	while (wsi) {
 		pt->fds[wsi->position_in_fds_table].revents |=
 			pt->fds[wsi->position_in_fds_table].events & LWS_POLLIN;
-		if (pt->fds[wsi->position_in_fds_table].revents &
-		    LWS_POLLIN)
+		if (pt->fds[wsi->position_in_fds_table].revents & LWS_POLLIN) {
 			forced = 1;
+			break;
+		}
 		wsi = wsi->u.ws.rx_draining_ext_list;
 	}
 
@@ -504,22 +549,23 @@ lws_http_client_read(struct lws *wsi, char **buf, int *len)
 {
 	int rlen, n;
 
-
-
 	rlen = lws_ssl_capable_read(wsi, (unsigned char *)*buf, *len);
-	if (rlen < 0)
-		return -1;
-
-	*len = rlen;
-	if (rlen == 0)
-		return 0;
-
-//	lwsl_err("%s: read %d\n", __func__, rlen);
+	*len = 0;
 
 	/* allow the source to signal he has data again next time */
-	wsi->client_rx_avail = 0;
 	lws_change_pollfd(wsi, 0, LWS_POLLIN);
 
+	if (rlen == LWS_SSL_CAPABLE_ERROR) {
+		lwsl_notice("%s: SSL capable error\n", __func__);
+		return -1;
+	}
+
+	if (rlen <= 0)
+		return 0;
+
+	*len = rlen;
+	wsi->client_rx_avail = 0;
+
 	/*
 	 * server may insist on transfer-encoding: chunked,
 	 * so http client must deal with it
@@ -533,14 +579,18 @@ spin_chunks:
 				break;
 			}
 			n = char_to_hex((*buf)[0]);
-			if (n < 0)
+			if (n < 0) {
+				lwsl_debug("chunking failure\n");
 				return -1;
+			}
 			wsi->chunk_remaining <<= 4;
 			wsi->chunk_remaining |= n;
 			break;
 		case ELCP_CR:
-			if ((*buf)[0] != '\x0a')
+			if ((*buf)[0] != '\x0a') {
+				lwsl_debug("chunking failure\n");
 				return -1;
+			}
 			wsi->chunk_parser = ELCP_CONTENT;
 			lwsl_info("chunk %d\n", wsi->chunk_remaining);
 			if (wsi->chunk_remaining)
@@ -552,8 +602,11 @@ spin_chunks:
 			break;
 
 		case ELCP_POST_CR:
-			if ((*buf)[0] != '\x0d')
+			if ((*buf)[0] != '\x0d') {
+				lwsl_debug("chunking failure\n");
+
 				return -1;
+			}
 
 			wsi->chunk_parser = ELCP_POST_LF;
 			break;
@@ -591,8 +644,11 @@ spin_chunks:
 #endif
 		if (user_callback_handle_rxflow(wsi->protocol->callback,
 				wsi, LWS_CALLBACK_RECEIVE_CLIENT_HTTP_READ,
-				wsi->user_space, *buf, n))
+				wsi->user_space, *buf, n)) {
+			lwsl_debug("%s: LWS_CALLBACK_RECEIVE_CLIENT_HTTP_READ returned -1\n", __func__);
+
 			return -1;
+		}
 
 	if (wsi->chunked && wsi->chunk_remaining) {
 		(*buf) += n;
@@ -603,52 +659,46 @@ spin_chunks:
 	if (wsi->chunked && !wsi->chunk_remaining)
 		wsi->chunk_parser = ELCP_POST_CR;
 
-	if (wsi->chunked && *len) {
+	if (wsi->chunked && *len)
 		goto spin_chunks;
-	}
 
 	if (wsi->chunked)
 		return 0;
 
-	wsi->u.http.content_remain -= n;
+	/* if we know the content length, decrement the content remaining */
+	if (wsi->u.http.content_length > 0)
+		wsi->u.http.content_remain -= n;
+
 	if (wsi->u.http.content_remain || !wsi->u.http.content_length)
 		return 0;
 
 completed:
 	if (user_callback_handle_rxflow(wsi->protocol->callback,
 			wsi, LWS_CALLBACK_COMPLETED_CLIENT_HTTP,
-			wsi->user_space, NULL, 0))
+			wsi->user_space, NULL, 0)) {
+		lwsl_debug("Completed call returned -1\n");
 		return -1;
+	}
 
-	if (lws_http_transaction_completed_client(wsi))
+	if (lws_http_transaction_completed_client(wsi)) {
+		lwsl_notice("%s: transaction completed says -1\n", __func__);
 		return -1;
+	}
 
 	return 0;
 }
 #endif
 
-/**
- * lws_service_fd() - Service polled socket with something waiting
- * @context:	Websocket context
- * @pollfd:	The pollfd entry describing the socket fd and which events
- *		happened.
- *
- *	This function takes a pollfd that has POLLIN or POLLOUT activity and
- *	services it according to the state of the associated
- *	struct lws.
- *
- *	The one call deals with all "service" that might happen on a socket
- *	including listen accepts, http files as well as websocket protocol.
- *
- *	If a pollfd says it has something, you can just pass it to
- *	lws_service_fd() whether it is a socket handled by lws or not.
- *	If it sees it is a lws socket, the traffic will be handled and
- *	pollfd->revents will be zeroed now.
- *
- *	If the socket is foreign to lws, it leaves revents alone.  So you can
- *	see if you should service yourself by checking the pollfd revents
- *	after letting lws try to service it.
- */
+static int
+lws_is_ws_with_ext(struct lws *wsi)
+{
+#if defined(LWS_NO_EXTENSIONS)
+	return 0;
+#else
+	return wsi->state == LWSS_ESTABLISHED &&
+	       !!wsi->count_act_ext;
+#endif
+}
 
 LWS_VISIBLE int
 lws_service_fd_tsi(struct lws_context *context, struct lws_pollfd *pollfd, int tsi)
@@ -667,20 +717,31 @@ lws_service_fd_tsi(struct lws_context *context, struct lws_pollfd *pollfd, int t
 	if (!context->protocol_init_done)
 		lws_protocol_init(context);
 
-	/*
-	 * you can call us with pollfd = NULL to just allow the once-per-second
-	 * global timeout checks; if less than a second since the last check
-	 * it returns immediately then.
-	 */
-
 	time(&now);
 
+	/*
+	 * handle case that system time was uninitialized when lws started
+	 * at boot, and got initialized a little later
+	 */
+	if (context->time_up < 1464083026 && now > 1464083026)
+		context->time_up = now;
+
 	/* TODO: if using libev, we should probably use timeout watchers... */
 	if (context->last_timeout_check_s != now) {
 		context->last_timeout_check_s = now;
 
 		lws_plat_service_periodic(context);
 
+		/* retire unused deprecated context */
+#if !defined(LWS_PLAT_OPTEE) && !defined(LWS_WITH_ESP32)
+#if LWS_POSIX && !defined(_WIN32)
+		if (context->deprecated && !context->count_wsi_allocated) {
+			lwsl_notice("%s: ending deprecated context\n", __func__);
+			kill(getpid(), SIGINT);
+			return 0;
+		}
+#endif
+#endif
 		/* global timeout check once per second */
 
 		if (pollfd)
@@ -690,7 +751,7 @@ lws_service_fd_tsi(struct lws_context *context, struct lws_pollfd *pollfd, int t
 		while (wsi) {
 			/* we have to take copies, because he may be deleted */
 			wsi1 = wsi->timeout_list;
-			tmp_fd = wsi->sock;
+			tmp_fd = wsi->desc.sockfd;
 			if (lws_service_timeout_check(wsi, (unsigned int)now)) {
 				/* he did time out... */
 				if (tmp_fd == our_fd)
@@ -717,6 +778,41 @@ lws_service_fd_tsi(struct lws_context *context, struct lws_pollfd *pollfd, int t
 #endif
 	}
 
+	/*
+	 * at intervals, check for ws connections needing ping-pong checks
+	 */
+
+	if (context->ws_ping_pong_interval &&
+	    context->last_ws_ping_pong_check_s < now + 10) {
+		struct lws_vhost *vh = context->vhost_list;
+		context->last_ws_ping_pong_check_s = now;
+
+		while (vh) {
+			for (n = 0; n < vh->count_protocols; n++) {
+				wsi = vh->same_vh_protocol_list[n];
+
+				while (wsi) {
+					if (wsi->state == LWSS_ESTABLISHED &&
+					    !wsi->socket_is_permanently_unusable &&
+					    !wsi->u.ws.send_check_ping &&
+					    wsi->u.ws.time_next_ping_check &&
+					    wsi->u.ws.time_next_ping_check < now) {
+
+						lwsl_info("requesting ping-pong on wsi %p\n", wsi);
+						wsi->u.ws.send_check_ping = 1;
+						lws_set_timeout(wsi, PENDING_TIMEOUT_WS_PONG_CHECK_SEND_PING,
+								context->timeout_secs);
+						lws_callback_on_writable(wsi);
+						wsi->u.ws.time_next_ping_check = now +
+								wsi->context->ws_ping_pong_interval;
+					}
+					wsi = wsi->same_vh_protocol_next;
+				}
+			}
+			vh = vh->vhost_next;
+		}
+	}
+
 	/* the socket we came to service timed out, nothing to do */
 	if (timed_out)
 		return 0;
@@ -755,7 +851,29 @@ lws_service_fd_tsi(struct lws_context *context, struct lws_pollfd *pollfd, int t
 
 #endif
 
-	lwsl_debug("fd=%d, revents=%d\n", pollfd->fd, pollfd->revents);
+//       lwsl_debug("fd=%d, revents=%d, mode=%d, state=%d\n", pollfd->fd, pollfd->revents, (int)wsi->mode, (int)wsi->state);
+	if (pollfd->revents & LWS_POLLHUP)
+		goto close_and_handled;
+
+
+#ifdef LWS_OPENSSL_SUPPORT
+       if ((wsi->state == LWSS_SHUTDOWN) && lws_is_ssl(wsi) && wsi->ssl)
+       {
+               n = SSL_shutdown(wsi->ssl);
+               lwsl_debug("SSH_shutdown=%d for fd %d\n", n, wsi->desc.sockfd);
+               if (n == 1)
+               {
+                       n = shutdown(wsi->desc.sockfd, SHUT_WR);
+                       goto close_and_handled;
+               }
+               else
+               {
+                       lws_change_pollfd(wsi, LWS_POLLOUT, LWS_POLLIN);
+                       n = 0;
+                       goto handled;
+               }
+       }
+#endif
 
 	/* okay, what we came here to do... */
 
@@ -765,6 +883,7 @@ lws_service_fd_tsi(struct lws_context *context, struct lws_pollfd *pollfd, int t
 	case LWSCM_HTTP_SERVING_ACCEPTED:
 	case LWSCM_SERVER_LISTENER:
 	case LWSCM_SSL_ACK_PENDING:
+	case LWSCM_SSL_ACK_PENDING_RAW:
 		if (wsi->state == LWSS_CLIENT_HTTP_ESTABLISHED)
 			goto handled;
 
@@ -776,11 +895,43 @@ lws_service_fd_tsi(struct lws_context *context, struct lws_pollfd *pollfd, int t
 			goto handled;
 		}
 #endif
+		/* fallthru */
+	case LWSCM_RAW:
 		n = lws_server_socket_service(context, wsi, pollfd);
 		if (n) /* closed by above */
 			return 1;
 		goto handled;
 
+	case LWSCM_RAW_FILEDESC:
+
+		if (pollfd->revents & LWS_POLLOUT) {
+			n = lws_calllback_as_writeable(wsi);
+			if (lws_change_pollfd(wsi, LWS_POLLOUT, 0)) {
+				lwsl_info("failed at set pollfd\n");
+				return 1;
+			}
+			if (n)
+				goto close_and_handled;
+		}
+		n = LWS_CALLBACK_RAW_RX;
+		if (wsi->mode == LWSCM_RAW_FILEDESC)
+			n = LWS_CALLBACK_RAW_RX_FILE;
+
+		if (pollfd->revents & LWS_POLLIN) {
+			if (user_callback_handle_rxflow(
+					wsi->protocol->callback,
+					wsi, n,
+					wsi->user_space, NULL, 0)) {
+				lwsl_debug("raw rx callback closed it\n");
+				goto close_and_handled;
+			}
+		}
+
+		if (pollfd->revents & LWS_POLLHUP)
+			goto close_and_handled;
+		n = 0;
+		goto handled;
+
 	case LWSCM_WS_SERVING:
 	case LWSCM_WS_CLIENT:
 	case LWSCM_HTTP2_SERVING:
@@ -789,11 +940,11 @@ lws_service_fd_tsi(struct lws_context *context, struct lws_pollfd *pollfd, int t
 		/* 1: something requested a callback when it was OK to write */
 
 		if ((pollfd->revents & LWS_POLLOUT) &&
-		    (wsi->state == LWSS_ESTABLISHED ||
+		    ((wsi->state == LWSS_ESTABLISHED ||
 		     wsi->state == LWSS_HTTP2_ESTABLISHED ||
 		     wsi->state == LWSS_HTTP2_ESTABLISHED_PRE_SETTINGS ||
 		     wsi->state == LWSS_RETURNED_CLOSE_ALREADY ||
-		     wsi->state == LWSS_FLUSHING_STORED_SEND_BEFORE_CLOSE) &&
+		     wsi->state == LWSS_FLUSHING_STORED_SEND_BEFORE_CLOSE)) &&
 		    lws_handle_POLLOUT_event(wsi, pollfd)) {
 			if (wsi->state == LWSS_RETURNED_CLOSE_ALREADY)
 				wsi->state = LWSS_FLUSHING_STORED_SEND_BEFORE_CLOSE;
@@ -891,9 +1042,24 @@ read:
 					wsi->u.hdr.ah->rxpos;
 		} else {
 			if (wsi->mode != LWSCM_HTTP_CLIENT_ACCEPTED) {
+				/*
+				 * extension may not consume everything (eg, pmd may be constrained
+				 * as to what it can output...) has to go in per-wsi rx buf area.
+				 * Otherwise in large temp serv_buf area.
+				 */
+				eff_buf.token = (char *)pt->serv_buf;
+				if (lws_is_ws_with_ext(wsi)) {
+					eff_buf.token_len = wsi->u.ws.rx_ubuf_alloc;
+				} else {
+					eff_buf.token_len = context->pt_serv_buf_size;
+				}
+
+				if (eff_buf.token_len > context->pt_serv_buf_size)
+					eff_buf.token_len = context->pt_serv_buf_size;
+
 				eff_buf.token_len = lws_ssl_capable_read(wsi,
-					pt->serv_buf, pending ? pending :
-							LWS_MAX_SOCKET_IO_BUF);
+					(unsigned char *)eff_buf.token, pending ? pending :
+					eff_buf.token_len);
 				switch (eff_buf.token_len) {
 				case 0:
 					lwsl_info("%s: zero length read\n", __func__);
@@ -906,16 +1072,23 @@ read:
 					lwsl_info("Closing when error\n");
 					goto close_and_handled;
 				}
-
-				eff_buf.token = (char *)pt->serv_buf;
+				// lwsl_notice("Actual RX %d\n", eff_buf.token_len);
 			}
 		}
 
 drain:
 #ifndef LWS_NO_CLIENT
-		if (wsi->mode == LWSCM_HTTP_CLIENT_ACCEPTED) {
+		if (wsi->mode == LWSCM_HTTP_CLIENT_ACCEPTED &&
+		    !wsi->told_user_closed) {
 
 			/*
+			 * In SSL mode we get POLLIN notification about
+			 * encrypted data in.
+			 *
+			 * But that is not necessarily related to decrypted
+			 * data out becoming available; in may need to perform
+			 * other in or out before that happens.
+			 *
 			 * simply mark ourselves as having readable data
 			 * and turn off our POLLIN
 			 */
@@ -923,13 +1096,15 @@ drain:
 			lws_change_pollfd(wsi, LWS_POLLIN, 0);
 
 			/* let user code know, he'll usually ask for writeable
-			 * callback and drain / reenable it there
+			 * callback and drain / re-enable it there
 			 */
 			if (user_callback_handle_rxflow(
 					wsi->protocol->callback,
 					wsi, LWS_CALLBACK_RECEIVE_CLIENT_HTTP,
-					wsi->user_space, NULL, 0))
+					wsi->user_space, NULL, 0)) {
+				lwsl_debug("LWS_CALLBACK_RECEIVE_CLIENT_HTTP closed it\n");
 				goto close_and_handled;
+			}
 		}
 #endif
 		/*
@@ -963,6 +1138,8 @@ drain:
 				 * around again it will pick up from where it
 				 * left off.
 				 */
+				// lwsl_notice("doing lws_read from pt->serv_buf %p %p for len %d\n", pt->serv_buf, eff_buf.token, (int)eff_buf.token_len);
+
 				n = lws_read(wsi, (unsigned char *)eff_buf.token,
 					     eff_buf.token_len);
 				if (n < 0) {
@@ -977,7 +1154,7 @@ drain:
 		} while (more);
 
 		if (wsi->u.hdr.ah) {
-			lwsl_info("%s: %p: detaching inherited used ah\n",
+			lwsl_notice("%s: %p: detaching\n",
 				 __func__, wsi);
 			/* show we used all the pending rx up */
 			wsi->u.hdr.ah->rxpos = wsi->u.hdr.ah->rxlen;
@@ -989,8 +1166,12 @@ drain:
 
 		pending = lws_ssl_pending(wsi);
 		if (pending) {
-			pending = pending > LWS_MAX_SOCKET_IO_BUF ?
-					LWS_MAX_SOCKET_IO_BUF : pending;
+			if (lws_is_ws_with_ext(wsi))
+				pending = pending > wsi->u.ws.rx_ubuf_alloc ?
+					wsi->u.ws.rx_ubuf_alloc : pending;
+			else
+				pending = pending > context->pt_serv_buf_size ?
+					context->pt_serv_buf_size : pending;
 			goto read;
 		}
 
@@ -1048,6 +1229,12 @@ drain:
 #ifdef LWS_NO_CLIENT
 		break;
 #else
+		if ((pollfd->revents & LWS_POLLOUT) &&
+		    lws_handle_POLLOUT_event(wsi, pollfd)) {
+			lwsl_debug("POLLOUT event closed it\n");
+			goto close_and_handled;
+		}
+
 		n = lws_client_socket_service(context, wsi, pollfd);
 		if (n)
 			return 1;
@@ -1079,38 +1266,6 @@ lws_service_fd(struct lws_context *context, struct lws_pollfd *pollfd)
 	return lws_service_fd_tsi(context, pollfd, 0);
 }
 
-/**
- * lws_service() - Service any pending websocket activity
- * @context:	Websocket context
- * @timeout_ms:	Timeout for poll; 0 means return immediately if nothing needed
- *		service otherwise block and service immediately, returning
- *		after the timeout if nothing needed service.
- *
- *	This function deals with any pending websocket traffic, for three
- *	kinds of event.  It handles these events on both server and client
- *	types of connection the same.
- *
- *	1) Accept new connections to our context's server
- *
- *	2) Call the receive callback for incoming frame data received by
- *	    server or client connections.
- *
- *	You need to call this service function periodically to all the above
- *	functions to happen; if your application is single-threaded you can
- *	just call it in your main event loop.
- *
- *	Alternatively you can fork a new process that asynchronously handles
- *	calling this service in a loop.  In that case you are happy if this
- *	call blocks your thread until it needs to take care of something and
- *	would call it with a large nonzero timeout.  Your loop then takes no
- *	CPU while there is nothing happening.
- *
- *	If you are calling it in a single-threaded app, you don't want it to
- *	wait around blocking other things in your loop from happening, so you
- *	would call it with a timeout_ms of 0, so it returns immediately if
- *	nothing is pending, or as soon as it services whatever was pending.
- */
-
 LWS_VISIBLE int
 lws_service(struct lws_context *context, int timeout_ms)
 {
@@ -1120,6 +1275,6 @@ lws_service(struct lws_context *context, int timeout_ms)
 LWS_VISIBLE int
 lws_service_tsi(struct lws_context *context, int timeout_ms, int tsi)
 {
-	return lws_plat_service_tsi(context, timeout_ms, tsi);
+	return _lws_plat_service_tsi(context, timeout_ms, tsi);
 }
 
