@@ -34,16 +34,10 @@
  *				using this protocol, including the sender
  */
 
-#if defined(LWS_USE_POLARSSL)
-#else
-#if defined(LWS_USE_MBEDTLS)
-#else
 #if defined(LWS_OPENSSL_SUPPORT) && defined(LWS_HAVE_SSL_CTX_set1_param)
 /* location of the certificate revocation list */
 extern char crl_path[1024];
 #endif
-#endif
-#endif
 
 extern int debug_level;
 
@@ -118,9 +112,66 @@ const char * get_mimetype(const char *file)
 	if (!strcmp(&file[n - 4], ".css"))
 		return "text/css";
 
+	if (!strcmp(&file[n - 3], ".js"))
+		return "text/javascript";
+
 	return NULL;
 }
 
+
+static const char * const param_names[] = {
+	"text",
+	"send",
+	"file",
+	"upload",
+};
+
+enum enum_param_names {
+	EPN_TEXT,
+	EPN_SEND,
+	EPN_FILE,
+	EPN_UPLOAD,
+};
+
+static int
+file_upload_cb(void *data, const char *name, const char *filename,
+	       char *buf, int len, enum lws_spa_fileupload_states state)
+{
+	struct per_session_data__http *pss =
+			(struct per_session_data__http *)data;
+	int n;
+
+	switch (state) {
+	case LWS_UFS_OPEN:
+		strncpy(pss->filename, filename, sizeof(pss->filename) - 1);
+		/* we get the original filename in @filename arg, but for
+		 * simple demo use a fixed name so we don't have to deal with
+		 * attacks  */
+		pss->post_fd = open("/tmp/post-file",
+			       O_CREAT | O_TRUNC | O_RDWR, 0600);
+		break;
+	case LWS_UFS_FINAL_CONTENT:
+	case LWS_UFS_CONTENT:
+		if (len) {
+			pss->file_length += len;
+
+			/* if the file length is too big, drop it */
+			if (pss->file_length > 100000)
+				return 1;
+
+			n = write(pss->post_fd, buf, len);
+			lwsl_notice("%s: write %d says %d\n", __func__, len, n);
+		}
+		if (state == LWS_UFS_CONTENT)
+			break;
+		close(pss->post_fd);
+		pss->post_fd = LWS_INVALID_FILE;
+		break;
+	}
+
+	return 0;
+}
+
 /* this protocol server (always the first one) handles HTTP,
  *
  * Some misc callbacks that aren't associated with a protocol also turn up only
@@ -133,7 +184,7 @@ int callback_http(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 	struct per_session_data__http *pss =
 			(struct per_session_data__http *)user;
 	unsigned char buffer[4096 + LWS_PRE];
-	unsigned long amount, file_len, sent;
+       lws_filepos_t amount, file_len, sent;
 	char leaf_path[1024];
 	const char *mimetype;
 	char *other_headers;
@@ -155,7 +206,7 @@ int callback_http(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 	switch (reason) {
 	case LWS_CALLBACK_HTTP:
 
-		lwsl_notice("lws_http_serve: %s\n",in);
+		lwsl_info("lws_http_serve: %s\n", (const char *)in);
 
 		if (debug_level & LLL_INFO) {
 			dump_handshake_info(wsi);
@@ -169,11 +220,8 @@ int callback_http(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 		}
 
 		{
-			char name[100], rip[50];
-			lws_get_peer_addresses(wsi, lws_get_socket_fd(wsi), name,
-					       sizeof(name), rip, sizeof(rip));
-			sprintf(buf, "%s (%s)", name, rip);
-			lwsl_notice("HTTP connect from %s\n", buf);
+			lws_get_peer_simple(wsi, buf, sizeof(buf));
+			lwsl_info("HTTP connect from %s\n", buf);
 		}
 
 		if (len < 1) {
@@ -226,39 +274,6 @@ int callback_http(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 		}
 #endif
 
-		if (!strncmp(in, "/postresults", 12)) {
-			m = sprintf(buf, "<html><body>Form results: '%s'<br>"
-					"</body></html>", pss->post_string);
-
-			p = buffer + LWS_PRE;
-			start = p;
-			end = p + sizeof(buffer) - LWS_PRE;
-
-			if (lws_add_http_header_status(wsi, 200, &p, end))
-				return 1;
-			if (lws_add_http_header_by_token(wsi,
-					WSI_TOKEN_HTTP_CONTENT_TYPE,
-				    	(unsigned char *)"text/html",
-					9, &p, end))
-				return 1;
-			if (lws_add_http_header_content_length(wsi, m, &p,
-							       end))
-				return 1;
-			if (lws_finalize_http_header(wsi, &p, end))
-				return 1;
-
-			n = lws_write(wsi, start, p - start,
-				      LWS_WRITE_HTTP_HEADERS);
-			if (n < 0)
-				return 1;
-
-			n = lws_write(wsi, (unsigned char *)buf, m, LWS_WRITE_HTTP);
-			if (n < 0)
-				return 1;
-
-			goto try_to_reuse;
-		}
-
 		/* if a legal POST URL, let it continue and accept data */
 		if (lws_hdr_total_length(wsi, WSI_TOKEN_POST_URI))
 			return 0;
@@ -266,6 +281,8 @@ int callback_http(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 		/* check for the "send a big file by hand" example case */
 
 		if (!strcmp((const char *)in, "/leaf.jpg")) {
+			lws_fop_flags_t flags = LWS_O_RDONLY;
+
 			if (strlen(resource_path) > sizeof(leaf_path) - 10)
 				return -1;
 			sprintf(leaf_path, "%s/leaf.jpg", resource_path);
@@ -275,13 +292,14 @@ int callback_http(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 			p = buffer + LWS_PRE;
 			end = p + sizeof(buffer) - LWS_PRE;
 
-			pss->fd = lws_plat_file_open(wsi, leaf_path, &file_len,
-						     LWS_O_RDONLY);
-
-			if (pss->fd == LWS_INVALID_FILE) {
-				lwsl_err("faild to open file %s\n", leaf_path);
+			pss->fop_fd = lws_vfs_file_open(
+					lws_get_fops(lws_get_context(wsi)),
+					leaf_path, &flags);
+			if (!pss->fop_fd) {
+				lwsl_err("failed to open file %s\n", leaf_path);
 				return -1;
 			}
+			file_len = lws_vfs_get_length(pss->fop_fd);
 
 			/*
 			 * we will send a big jpeg file, but it could be
@@ -293,7 +311,7 @@ int callback_http(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 			 * depending on what connection it happens to be working
 			 * on
 			 */
-			if (lws_add_http_header_status(wsi, 200, &p, end))
+			if (lws_add_http_header_status(wsi, HTTP_STATUS_OK, &p, end))
 				return 1;
 			if (lws_add_http_header_by_token(wsi, WSI_TOKEN_HTTP_SERVER,
 				    	(unsigned char *)"libwebsockets",
@@ -329,7 +347,7 @@ int callback_http(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 				      p - (buffer + LWS_PRE),
 				      LWS_WRITE_HTTP_HEADERS);
 			if (n < 0) {
-				lws_plat_file_close(wsi, pss->fd);
+				lws_vfs_file_close(&pss->fop_fd);
 				return -1;
 			}
 			/*
@@ -404,29 +422,86 @@ int callback_http(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 		break;
 
 	case LWS_CALLBACK_HTTP_BODY:
-		lwsl_notice("LWS_CALLBACK_HTTP_BODY: len %d\n", (int)len);
-		strncpy(pss->post_string, in, sizeof (pss->post_string) -1);
-		pss->post_string[sizeof(pss->post_string) - 1] = '\0';
-		if (len < sizeof(pss->post_string) - 1)
-			pss->post_string[len] = '\0';
+		/* create the POST argument parser if not already existing */
+		if (!pss->spa) {
+			pss->spa = lws_spa_create(wsi, param_names,
+					ARRAY_SIZE(param_names), 1024,
+					file_upload_cb, pss);
+			if (!pss->spa)
+				return -1;
+
+			pss->filename[0] = '\0';
+			pss->file_length = 0;
+		}
+
+		/* let it parse the POST data */
+		if (lws_spa_process(pss->spa, in, len))
+			return -1;
 		break;
 
 	case LWS_CALLBACK_HTTP_BODY_COMPLETION:
-		lwsl_notice("LWS_CALLBACK_HTTP_BODY_COMPLETION\n");
+		lwsl_debug("LWS_CALLBACK_HTTP_BODY_COMPLETION\n");
 		/*
 		 * the whole of the sent body arrived,
 		 * respond to the client with a redirect to show the
 		 * results
 		 */
-		p = (unsigned char *)buf + LWS_PRE;
-		n = lws_http_redirect(wsi,
-				      HTTP_STATUS_SEE_OTHER, /* 303 */
-				      (unsigned char *)"/postresults", 12, /* location + len */
-				      &p, /* temp buffer to use */
-				      p + sizeof(buf) - 1 - LWS_PRE /* buffer len */
-			);
-		goto try_to_reuse;
 
+		/* call to inform no more payload data coming */
+		lws_spa_finalize(pss->spa);
+
+		p = (unsigned char *)pss->result + LWS_PRE;
+		end = p + sizeof(pss->result) - LWS_PRE - 1;
+		p += sprintf((char *)p,
+			"<html><body><h1>Form results (after urldecoding)</h1>"
+			"<table><tr><td>Name</td><td>Length</td><td>Value</td></tr>");
+
+		for (n = 0; n < ARRAY_SIZE(param_names); n++)
+			p += lws_snprintf((char *)p, end - p,
+				    "<tr><td><b>%s</b></td><td>%d</td><td>%s</td></tr>",
+				    param_names[n],
+				    lws_spa_get_length(pss->spa, n),
+				    lws_spa_get_string(pss->spa, n));
+
+		p += lws_snprintf((char *)p, end - p, "</table><br><b>filename:</b> %s, <b>length</b> %ld",
+				pss->filename, pss->file_length);
+
+		p += lws_snprintf((char *)p, end - p, "</body></html>");
+		pss->result_len = p - (unsigned char *)(pss->result + LWS_PRE);
+
+		p = buffer + LWS_PRE;
+		start = p;
+		end = p + sizeof(buffer) - LWS_PRE;
+
+		if (lws_add_http_header_status(wsi, HTTP_STATUS_OK, &p, end))
+			return 1;
+
+		if (lws_add_http_header_by_token(wsi, WSI_TOKEN_HTTP_CONTENT_TYPE,
+				(unsigned char *)"text/html", 9, &p, end))
+			return 1;
+		if (lws_add_http_header_content_length(wsi, pss->result_len, &p, end))
+			return 1;
+		if (lws_finalize_http_header(wsi, &p, end))
+			return 1;
+
+		n = lws_write(wsi, start, p - start, LWS_WRITE_HTTP_HEADERS);
+		if (n < 0)
+			return 1;
+
+		n = lws_write(wsi, (unsigned char *)pss->result + LWS_PRE,
+			      pss->result_len, LWS_WRITE_HTTP);
+		if (n < 0)
+			return 1;
+		goto try_to_reuse;
+	case LWS_CALLBACK_HTTP_DROP_PROTOCOL:
+		lwsl_debug("LWS_CALLBACK_HTTP_DROP_PROTOCOL\n");
+
+		/* called when our wsi user_space is going to be destroyed */
+		if (pss->spa) {
+			lws_spa_destroy(pss->spa);
+			pss->spa = NULL;
+		}
+		break;
 	case LWS_CALLBACK_HTTP_FILE_COMPLETION:
 		goto try_to_reuse;
 
@@ -436,7 +511,7 @@ int callback_http(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 		if (pss->client_finished)
 			return -1;
 
-		if (pss->fd == LWS_INVALID_FILE)
+		if (!pss->fop_fd)
 			goto try_to_reuse;
 
 #ifndef LWS_NO_CLIENT
@@ -481,7 +556,7 @@ int callback_http(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 				/* he couldn't handle that much */
 				n = m;
 
-			n = lws_plat_file_read(wsi, pss->fd,
+			n = lws_vfs_file_read(pss->fop_fd,
 					       &amount, buffer + LWS_PRE, n);
 			/* problem reading, close conn */
 			if (n < 0) {
@@ -514,12 +589,12 @@ later:
 		lws_callback_on_writable(wsi);
 		break;
 penultimate:
-		lws_plat_file_close(wsi, pss->fd);
-		pss->fd = LWS_INVALID_FILE;
+		lws_vfs_file_close(&pss->fop_fd);
+		pss->fop_fd = NULL;
 		goto try_to_reuse;
 
 bail:
-		lws_plat_file_close(wsi, pss->fd);
+		lws_vfs_file_close(&pss->fop_fd);
 
 		return -1;
 
@@ -540,7 +615,7 @@ bail:
 		lwsl_err("LWS_CALLBACK_ESTABLISHED_CLIENT_HTTP\n");
 		p = buffer + LWS_PRE;
 		end = p + sizeof(buffer) - LWS_PRE;
-		if (lws_add_http_header_status(lws_get_parent(wsi), 200, &p, end))
+		if (lws_add_http_header_status(lws_get_parent(wsi), HTTP_STATUS_OK, &p, end))
 			return 1;
 		if (lws_add_http_header_by_token(lws_get_parent(wsi),
 				WSI_TOKEN_HTTP_SERVER,
@@ -671,10 +746,6 @@ bail:
 
 		break;
 
-#if defined(LWS_USE_POLARSSL)
-#else
-#if defined(LWS_USE_MBEDTLS)
-#else
 #if defined(LWS_OPENSSL_SUPPORT)
 	case LWS_CALLBACK_OPENSSL_PERFORM_CLIENT_CERT_VERIFICATION:
 		/* Verify the client certificate */
@@ -706,8 +777,6 @@ bail:
 		}
 		break;
 #endif
-#endif
-#endif
 #endif
 
 	default:
