@@ -30,15 +30,9 @@ struct lws_plat_file_ops fops_plat;
 #define LOCAL_RESOURCE_PATH INSTALL_DATADIR"/libwebsockets-test-server"
 char *resource_path = LOCAL_RESOURCE_PATH;
 
-#if defined(LWS_USE_POLARSSL)
-#else
-#if defined(LWS_USE_MBEDTLS)
-#else
 #if defined(LWS_OPENSSL_SUPPORT) && defined(LWS_HAVE_SSL_CTX_set1_param)
 char crl_path[1024] = "";
 #endif
-#endif
-#endif
 
 /*
  * libev dumps their hygiene problems on their users blaming compiler
@@ -128,6 +122,12 @@ static struct lws_protocols protocols[] = {
 		sizeof(struct per_session_data__lws_mirror),
 		128,
 	},
+	{
+		"lws-status",
+		callback_lws_status,
+		sizeof(struct per_session_data__lws_status),
+		128,
+	},
 	{ NULL, NULL, 0, 0 } /* terminator */
 };
 
@@ -149,17 +149,17 @@ static const struct lws_extension exts[] = {
  * to do any of this unless you have a reason (eg, want to serve
  * compressed files without decompressing the whole archive)
  */
-static lws_filefd_type
-test_server_fops_open(struct lws *wsi, const char *filename,
-		      unsigned long *filelen, int flags)
+static lws_fop_fd_t
+test_server_fops_open(const struct lws_plat_file_ops *fops,
+		      const char *vfs_path, const char *vpath,
+		      lws_fop_flags_t *flags)
 {
-	lws_filefd_type n;
+	lws_fop_fd_t n;
 
 	/* call through to original platform implementation */
-	n = fops_plat.open(wsi, filename, filelen, flags);
+	n = fops_plat.open(fops, vfs_path, vpath, flags);
 
-	lwsl_notice("%s: opening %s, ret %ld, len %lu\n", __func__, filename,
-			(long)n, *filelen);
+	lwsl_notice("%s: opening %s, ret %p\n", __func__, vfs_path, n);
 
 	return n;
 }
